/*****************************************************************************
* Filename:          xps_sil00.h
* Version:           1.00.b
* Description:       xps_sil00 Driver Header File
* Date:              Mon Aug 09 13:09:55 2010 (by Create and Import Peripheral Wizard)
*****************************************************************************/

#ifndef XPS_SIL00_H
#define XPS_SIL00_H

/***************************** Include Files *******************************/

#include "xbasic_types.h"
#include "xstatus.h"
#include "xio.h"

/************************** Constant Definitions ***************************/


/**
 * User Logic Slave Space Offsets
 * -- SLV_REG0 : user logic slave module register 0
 * -- SLV_REG1 : user logic slave module register 1
 */
#define XPS_SIL00_USER_SLV_SPACE_OFFSET (0x00000000)
#define XPS_SIL00_SLV_REG0_OFFSET (XPS_SIL00_USER_SLV_SPACE_OFFSET + 0x00000000)
#define XPS_SIL00_SLV_REG1_OFFSET (XPS_SIL00_USER_SLV_SPACE_OFFSET + 0x00000004)

/**
 * Interrupt Controller Space Offsets
 * -- INTR_DGIER : device (peripheral) global interrupt enable register
 * -- INTR_ISR   : ip (user logic) interrupt status register
 * -- INTR_IER   : ip (user logic) interrupt enable register
 */
#define XPS_SIL00_INTR_CNTRL_SPACE_OFFSET (0x00000100)
#define XPS_SIL00_INTR_DGIER_OFFSET (XPS_SIL00_INTR_CNTRL_SPACE_OFFSET + 0x0000001C)
#define XPS_SIL00_INTR_IPISR_OFFSET (XPS_SIL00_INTR_CNTRL_SPACE_OFFSET + 0x00000020)
#define XPS_SIL00_INTR_IPIER_OFFSET (XPS_SIL00_INTR_CNTRL_SPACE_OFFSET + 0x00000028)

/**
 * Interrupt Controller Masks
 * -- INTR_TERR_MASK : transaction error
 * -- INTR_DPTO_MASK : data phase time-out
 * -- INTR_IPIR_MASK : ip interrupt requeset
 * -- INTR_RFDL_MASK : read packet fifo deadlock interrupt request
 * -- INTR_WFDL_MASK : write packet fifo deadlock interrupt request
 * -- INTR_IID_MASK  : interrupt id
 * -- INTR_GIE_MASK  : global interrupt enable
 * -- INTR_NOPEND    : the DIPR has no pending interrupts
 */
#define INTR_TERR_MASK (0x00000001UL)
#define INTR_DPTO_MASK (0x00000002UL)
#define INTR_IPIR_MASK (0x00000004UL)
#define INTR_RFDL_MASK (0x00000020UL)
#define INTR_WFDL_MASK (0x00000040UL)
#define INTR_IID_MASK (0x000000FFUL)
#define INTR_GIE_MASK (0x80000000UL)
#define INTR_NOPEND (0x80)

/**************************** Type Definitions *****************************/


/***************** Macros (Inline Functions) Definitions *******************/

/**
 *
 * Write a value to a XPS_SIL00 register. A 32 bit write is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is written.
 *
 * @param   BaseAddress is the base address of the XPS_SIL00 device.
 * @param   RegOffset is the register offset from the base to write to.
 * @param   Data is the data written to the register.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void XPS_SIL00_mWriteReg(Xuint32 BaseAddress, unsigned RegOffset, Xuint32 Data)
 *
 */
#define XPS_SIL00_mWriteReg(BaseAddress, RegOffset, Data) \
 	XIo_Out32((BaseAddress) + (RegOffset), (Xuint32)(Data))

/**
 *
 * Read a value from a XPS_SIL00 register. A 32 bit read is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is read from the register. The most significant data
 * will be read as 0.
 *
 * @param   BaseAddress is the base address of the XPS_SIL00 device.
 * @param   RegOffset is the register offset from the base to write to.
 *
 * @return  Data is the data from the register.
 *
 * @note
 * C-style signature:
 * 	Xuint32 XPS_SIL00_mReadReg(Xuint32 BaseAddress, unsigned RegOffset)
 *
 */
#define XPS_SIL00_mReadReg(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (RegOffset))


/**
 *
 * Write/Read 32 bit value to/from XPS_SIL00 user logic slave registers.
 *
 * @param   BaseAddress is the base address of the XPS_SIL00 device.
 * @param   RegOffset is the offset from the slave register to write to or read from.
 * @param   Value is the data written to the register.
 *
 * @return  Data is the data from the user logic slave register.
 *
 * @note
 * C-style signature:
 * 	void XPS_SIL00_mWriteSlaveRegn(Xuint32 BaseAddress, unsigned RegOffset, Xuint32 Value)
 * 	Xuint32 XPS_SIL00_mReadSlaveRegn(Xuint32 BaseAddress, unsigned RegOffset)
 *
 */
#define XPS_SIL00_mWriteSlaveReg0(BaseAddress, RegOffset, Value) \
 	XIo_Out32((BaseAddress) + (XPS_SIL00_SLV_REG0_OFFSET) + (RegOffset), (Xuint32)(Value))
#define XPS_SIL00_mWriteSlaveReg1(BaseAddress, RegOffset, Value) \
 	XIo_Out32((BaseAddress) + (XPS_SIL00_SLV_REG1_OFFSET) + (RegOffset), (Xuint32)(Value))

#define XPS_SIL00_mReadSlaveReg0(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (XPS_SIL00_SLV_REG0_OFFSET) + (RegOffset))
#define XPS_SIL00_mReadSlaveReg1(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (XPS_SIL00_SLV_REG1_OFFSET) + (RegOffset))

/************************** Function Prototypes ****************************/


/**
 *
 * Enable all possible interrupts from XPS_SIL00 device.
 *
 * @param   baseaddr_p is the base address of the XPS_SIL00 device.
 *
 * @return  None.
 *
 * @note    None.
 *
 */
void XPS_SIL00_EnableInterrupt(void * baseaddr_p);

/**
 *
 * Example interrupt controller handler.
 *
 * @param   baseaddr_p is the base address of the XPS_SIL00 device.
 *
 * @return  None.
 *
 * @note    None.
 *
 */
void XPS_SIL00_Intr_DefaultHandler(void * baseaddr_p);

/**
 *
 * Run a self-test on the driver/device. Note this may be a destructive test if
 * resets of the device are performed.
 *
 * If the hardware system is not built correctly, this function may never
 * return to the caller.
 *
 * @param   baseaddr_p is the base address of the XPS_SIL00 instance to be worked on.
 *
 * @return
 *
 *    - XST_SUCCESS   if all self-test code passed
 *    - XST_FAILURE   if any self-test code failed
 *
 * @note    Caching must be turned off for this function to work.
 * @note    Self test may fail if data memory and device are not on the same bus.
 *
 */
XStatus XPS_SIL00_SelfTest(void * baseaddr_p);

#endif /** XPS_SIL00_H */
