-------------------------------------------------------------------------------
-- Filename:       dynamic_ctrl.vhd
-- History:
--   2006-7-14     First Version
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity dynamic_ctrl is
  port (
    SYS_CLK     : in  std_logic;
    SYS_RST     : in  std_logic;
    nSEL        : out std_logic_vector(0 to 2);
    seg7_timing : in  std_logic;
    seg_in1     : in  std_logic_vector(0 to 3);
    seg_in2     : in  std_logic_vector(0 to 3);
    seg_in3     : in  std_logic_vector(0 to 3);
    seg_data    : out std_logic_vector(0 to 3)
  );
end dynamic_ctrl;

architecture imp of dynamic_ctrl is

  signal sel          : std_logic_vector(0 to 2);
  signal seg_data_w   : std_logic_vector(0 to 3);
  signal seg7_tim_reg : std_logic;

begin

  process(SYS_CLK)
  begin
    if SYS_CLK'event and SYS_CLK = '1' then
      if SYS_RST = '1' then
        seg7_tim_reg <= '0';
      else
        seg7_tim_reg <= seg7_timing;
      end if;
    end if;
  end process;

  process(SYS_CLK)
  begin
    if SYS_CLK'event and SYS_CLK = '1' then
      if SYS_RST = '1' then
        sel <= "001";
      else
        if seg7_timing = '1' and seg7_tim_reg = '0' then
          sel <= sel(1 to 2) & sel(0);
        end if;
      end if;
    end if;
  end process;

  seg_data_w <= seg_in1 when sel = "001" else seg_in2 when sel = "010" else seg_in3;

  nSEL <= not sel;
  seg_data <= seg_data_w;

end imp;

